/*******************************************************************
*
*  DESCRIPTION: class Analizer
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 26/04/2003
*
*******************************************************************/

/** include files **/
#include "analizer.h"   // base class
#include "message.h"    // class ExternalMessage 
#include "mainsimu.h"   // class MainSimulator

/*******************************************************************
* Function Name: Transducer
********************************************************************/
Analizer::Analizer (const string &name)
: Atomic (name) 
, arrived (addInputPort ("arrived"))
, solved (addInputPort ("solved"))
, stop (addInputPort ("stop"))
, average (addOutputPort ("average")) 
, quantity (addOutputPort ("quantity")) {
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &Analizer::initFunction () {
    totElapsed = Time::Zero;
    arrivedCount = solvedCount = 0;
    unsolved.erase (unsolved.begin (), unsolved.end ());
    hasToStop = false;
    
    // Estoy siempre activo esperando computar eventos...
    holdIn (active, Time::Inf);
    return *this;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &Analizer::externalFunction (const ExternalMessage &msg) {
    if (state () == active) {
        // Analizo la info solo si estoy activo....
        if (msg.port () == arrived) {
            // Verifico que no este duplicado....
            if (unsolved.find (msg.value ()) != unsolved.end ()) {
                MException e (string ("Unresolved Work Id: ") + msg.value () + " is duplicated.");
                e.addLocation (MEXCEPTION_LOCATION ());
                throw e;
            }
            // Lo cuento como llegado, y almaceno su hora de llegada.
            arrivedCount++;
            unsolved [msg.value ()] = msg.time ();
        }

        if (msg.port () == stop) {
            // scheduleo el stop para cuando no pasenmas vehiculos.
            hasToStop = true;
            if (arrivedCount == solvedCount) {
                holdIn (active, Time::Zero);
            }   
        }
        
        if (msg.port () == solved) {
            // Chequeo que sea uno de los que llegaron....
            JobsList::iterator cursor (unsolved.find (msg.value ()));
            if (cursor == unsolved.end ()) {
                MException e (string ("Resolved Work Id: ") + msg.value () + " Not Found!");
                e.addLocation (MEXCEPTION_LOCATION ());
                throw e;
            }
            // Lo cuento como resuleto, y sumo si tiempo "en transito" para
            // luego calcular el promedio.
            solvedCount++;
            totElapsed += (msg.time () - unsolved [msg.value ()]);
            unsolved.erase (cursor);
            // si la cantidad de llegados y resueltos es la misma, significa que
            // tengo que generar la salida con el promedio.
            if (hasToStop && (arrivedCount == solvedCount)) {
                holdIn (active, Time::Zero);
            }   
        }
    } else {
        // no hago nada, porque ya termine de procesar las estadisticas.... 
    }   
    return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &Analizer::internalFunction (const InternalMessage &) {
    if (state () == active) {
        // Termino la ejecucion....
        passivate ();
    }
    return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &Analizer::outputFunction (const InternalMessage &msg) {
    
    long avg;
    
    avg = long (totElapsed.asMsecs () / solvedCount);
    sendOutput (msg.time (), quantity, solvedCount);
    sendOutput (msg.time (), average, avg);

    return *this ;
}
