/*******************************************************************
*
*  DESCRIPTION: class TimeControlUnit
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 02/05/2003
*
*******************************************************************/

/** include files **/
#include "time_cu.h"       // base header
#include "message.h"       // class InternalMessage 
#include "mainsimu.h"      // class Simulator
#include "distri.h"        // class Distribution 
#include "strutil.h"       // str2Value( ... )

/*******************************************************************
* Function Name: TimeControlUnit
********************************************************************/
TimeControlUnit::TimeControlUnit (const string &name)
: Atomic (name)
, in (addInputPort ("in"))
, out (addInputPort ("out")) 
, opens (addOutputPort ("opens")) 
, openn (addOutputPort ("openn")) {
	if (MainSimulator::Instance ().existsParameter (description (), "max_north")) {
		max_north = MainSimulator::Instance ().getParameter (description (), "max_north");
	} else {
		// por default, 1 minuto.
		max_north = Time (0, 1, 0, 0);
	}
	if (MainSimulator::Instance ().existsParameter (description (), "max_south")) {
		max_south = MainSimulator::Instance ().getParameter (description (), "max_south");
	} else {
		// por default, 1 minuto.
		max_south = Time (0, 1, 0, 0);
	}
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &TimeControlUnit::initFunction () {
	in_transit = 0;
    phase = OPEN_NORTH;
    holdIn (active, Time::Zero);
    return *this;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &TimeControlUnit::externalFunction (const ExternalMessage &msg) {
    if (msg.port () == in) {
        in_transit++;
    }
    if (msg.port () == out) {
        in_transit--;
        // si no tengo mas autos y estaba liberando, corto el Infinito
        // para que la funcion interna decida a que estado pasar....
        if ((in_transit == 0) && (phase == FREE_NORTH || phase == FREE_SOUTH)) {
       		holdIn (active, Time::Zero);
        }	
    }
    return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &TimeControlUnit::internalFunction (const InternalMessage &) {
	switch (phase) {
		case OPEN_NORTH:
			phase = IN_NORTH;
		 	holdIn (active, max_north);
			break;
		case IN_NORTH:
			phase = CLOSE_NORTH;
		 	holdIn (active, Time::Zero);
			break;
		case CLOSE_NORTH:
			// si tengo autos, me quedo esperando que salgan todos.
			if (in_transit > 0) {
				phase = FREE_NORTH;			
		 		holdIn (active, Time::Inf);
		 	} else {
		 		phase = OPEN_SOUTH;
			 	holdIn (active, Time::Zero);
			}
			break;
		case FREE_NORTH:
			phase = OPEN_SOUTH;
		 	holdIn (active, Time::Zero);
			break;
		case OPEN_SOUTH:
			phase = IN_SOUTH;
		 	holdIn (active, max_south);
			break;
		case IN_SOUTH:
			phase = CLOSE_SOUTH;
		 	holdIn (active, Time::Zero);
			break;
		case CLOSE_SOUTH:
			// si tengo autos, me quedo esperando que salgan todos.
			if (in_transit > 0) {
				phase = FREE_SOUTH;
			 	holdIn (active, Time::Inf);
		 	} else {
		 		phase = OPEN_NORTH;
			 	holdIn (active, Time::Zero);
			}
			break;
		case FREE_SOUTH:
			phase = OPEN_NORTH;
		 	holdIn (active, Time::Zero);
			break;
	}
    return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &TimeControlUnit::outputFunction (const InternalMessage &msg) {
	switch (phase) {
		case OPEN_NORTH:
		    sendOutput (msg.time (), openn, 1);
			break;
		case IN_NORTH:
			// No tengo que hacer nada...
			break;
		case CLOSE_NORTH:
		    sendOutput (msg.time (), openn, 0);
			break;
		case FREE_NORTH:
			// No tengo que hacer nada...
			break;
		case OPEN_SOUTH:
		    sendOutput (msg.time (), opens, 1);
			break;
		case IN_SOUTH:
			// No tengo que hacer nada...
			break;
		case CLOSE_SOUTH:
		    sendOutput (msg.time (), opens, 0);
			break;
		case FREE_SOUTH:
			// No tengo que hacer nada...
			break;
	}
    return *this;
}

